from setuptools import setup

setup(name='zmpe',
      version='1.0.1',
      description='The program controls the execution of any programs, scripts or commands OS and sends the execution result to zabbix, and in case of an execution error, it additionally can notify via telegram.',
      packages=['zmpe'],
      author_email='ministrbob777@gmail.com',
      zip_safe=False)
